import json
from collections import OrderedDict
import csv

CSV_SETTINGS = dict(delimiter=',', quotechar='$', quoting=csv.QUOTE_MINIMAL)

WEIGHT_MATRIX = [1, 1, 1]


def parse_file(file_path):
    with open(file_path) as _file:
        content = _file.read()

    print("Parsing...", end="\r")
    total_lines = len(content.splitlines())

    if content.startswith("["):
        return json.loads(content)

    new_content = []
    for i, line in enumerate(content.splitlines()):
        print("Parsing... line %s of %s" % (i, total_lines), end="\r")
        new_content.append(line + ("," if i != total_lines-1 else ""))

    content = "\n".join(new_content)
    content = "[%s]" % content
    print("Parsing complete" + " "*20)

    with open(file_path, "w") as _file:
        _file.write(content)

    print("Loading JSON...", end="\r")
    json_content = json.loads(content)

    with open(file_path, "w") as _file:
        _file.write(json.dumps(json_content, indent=2))
    print("Loading JSON complete")

    print("Sorting list...", end="\r")
    json_content.sort(key=lambda entry: entry["eulerAngles"]["y"])
    print("Sorting list complete")

    return json_content


def calculate_score(_object):
    score = 0

    values_matrix = [
        _object.get("angle", None),
        _object.get("distance", None),
        _object.get("volume", None)
    ]

    for i, value in enumerate(values_matrix):
        if value is None:
            continue

        v = 1/value

        score += v * WEIGHT_MATRIX[i]

    return score


def score_sample(sample_point):
    local_scores = {}

    for _object in sample_point["objects"]:
        local_scores[_object["name"]] = calculate_score(_object)

    return local_scores


def score_trace(json_data):
    total_scores = {}

    for sample_point in json_data:
        local_scores = score_sample(sample_point)

        for k, v in local_scores.items():
            if "terrain" in k.lower():
                continue
            if k in total_scores:
                total_scores[k] += v
            else:
                total_scores[k] = v
    
    # Unnecessary objects
    total_scores.pop("Terrain", None)
    total_scores.pop("Plane", None)
    total_scores.pop("OVRPlayerController", None)

    return total_scores


folder = "/Users/Daniel/Desktop/TDDD82/results/"
file = "viking_1.json"

json_data = parse_file(folder + file)

scores = score_trace(json_data)

sorted_scores = sorted(scores.items(), key=lambda x: x[1], reverse=True)

file_name = "%s%s_%s.csv"\
            % (folder, "".join(file.split(".")[0:-1]), WEIGHT_MATRIX)

with open(file_name, "w") as csv_file:
    writer = csv.writer(csv_file, **CSV_SETTINGS)
    writer.writerow(["Object", "Score"])

    for line in sorted_scores:
        writer.writerow(line)
